require 'puddle/version'
require "puddle/cli"
require "puddle/http_adapter"
require "puddle/resources"
require 'pp'
module Puddle
  class << self

    def validate_token(settings = {})
      token = settings['token'] || ENV['DO_API_TOKEN']
      fail 'api token is required but missing' unless token
      token
    end

    def create_adapter(token, settings = {})
      class_constant = settings['http_adapter_class'] || HttpAdapter
      class_constant.new(token, settings)
    end

    def load_resource(name)
      class_name  = name.to_s.split(/_/).map{|word| word.capitalize}.join('')

      Puddle::Resources.const_get(class_name)
    end

    def digital_ocean(name, settings = {})
      results = {}
      begin
        token    = validate_token(settings)
        adapter  = create_adapter(token, settings)
        resource = load_resource(name).new(adapter, settings)
        results[:content] = resource.execute
      rescue Exception => e
        results[:error] = e
      end

      return results
    end
  end
end
