require 'rest-client'
require 'json'

module Puddle
  class HttpAdapter
    DO_API_BASE     = 'https://api.digitalocean.com'
    DO_API_VERSION  = 'v2'

    attr_reader :token, :headers, :timeout

    def self.digital_ocean_api_url
      "#{DO_API_BASE}/#{DO_API_VERSION}"
    end

    def self.endpoint_url(endpoint)
      "#{digital_ocean_api_url}/#{endpoint}"
    end

    def initialize(api_token, options = {})
      @token   = api_token
      @headers = build_headers(options[:headers] || {})
      @timeout = options[:timeout] || 10
    end

    def content_type_header
      'application/json'
    end

    def accept_header
      '*/*; q=0.5, application/json'
    end

    def authorization_header
      "Bearer #{token}"
    end

    def user_agent_header
      'Puddle'
    end

    def accept_encoding_header
      'gzip, deflate'
    end

    def build_headers(headers = {})
      defaults = {
        'Accept'          => accept_header,
        'Accept-Encoding' => accept_encoding_header,
        'Authorization'   => authorization_header,
        'Content-Type'    => content_type_header,
        'User-Agent'      => user_agent_header,
      }

      defaults.merge(headers)
    end

    def api(method, endpoint, options = {})
      headers = build_headers(options['headers'] || {})
      result = RestClient::Request.execute(
        method: method,
        url: self.class.endpoint_url(endpoint),
        headers: headers,
        timeout: timeout
      )

      return result if options['format'] == 'json'
      JSON.parse(result)
    end
  end
end
